import processing.core.*; 
import processing.xml.*; 

import processing.video.*; 
import jp.nyatla.nyar4psg.*; 
import fullscreen.*; 

import jp.nyatla.nyartoolkit.core.*; 
import jp.nyatla.nyartoolkit.core.labeling.artoolkit.*; 
import jp.nyatla.nyartoolkit.utils.*; 
import jp.nyatla.nyartoolkit.core.raster.*; 
import jp.nyatla.nyartoolkit.core.types.stack.*; 
import jp.nyatla.nyartoolkit.core.utils.*; 
import jp.nyatla.nyartoolkit.core.pickup.*; 
import jp.nyatla.nyartoolkit.core.rasterfilter.rgb2bin.*; 
import jp.nyatla.nyartoolkit.core.labeling.rlelabeling.*; 
import jp.nyatla.nyartoolkit.core.analyzer.raster.*; 
import jp.nyatla.nyartoolkit.core.rasterfilter.rgb2gs.*; 
import jp.nyatla.nyartoolkit.core.match.*; 
import jp.nyatla.nyartoolkit.core.param.*; 
import jp.nyatla.nyartoolkit.sample.*; 
import jp.nyatla.nyartoolkit.core.types.matrix.*; 
import jp.nyatla.nyartoolkit.core.transmat.optimize.*; 
import jp.nyatla.nyartoolkit.core.transmat.rotmatrix.*; 
import jp.nyatla.nyartoolkit.nyidmarker.data.*; 
import jp.nyatla.nyartoolkit.core.rasterfilter.gs2bin.*; 
import jp.nyatla.nyartoolkit.core.squaredetect.*; 
import jp.nyatla.nyartoolkit.detector.*; 
import jp.nyatla.nyartoolkit.core.rasterreader.*; 
import jp.nyatla.nyartoolkit.core.transmat.solver.*; 
import jp.nyatla.nyartoolkit.core.pca2d.*; 
import jp.nyatla.nyartoolkit.core.raster.rgb.*; 
import jp.nyatla.nyartoolkit.core.analyzer.raster.threshold.*; 
import jp.nyatla.nyartoolkit.nyidmarker.*; 
import jp.nyatla.nyartoolkit.core.labeling.*; 
import jp.nyatla.nyartoolkit.utils.j2se.*; 
import jp.nyatla.nyartoolkit.core.transmat.optimize.artoolkit.*; 
import jp.nyatla.nyar4psg.*; 
import jp.nyatla.nyartoolkit.core.analyzer.histogram.*; 
import jp.nyatla.nyartoolkit.core.rasterfilter.*; 
import jp.nyatla.nyartoolkit.core.transmat.*; 
import jp.nyatla.nyartoolkit.*; 
import jp.nyatla.nyartoolkit.core.types.*; 
import jp.nyatla.nyartoolkit.processor.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class AR_pendulum extends PApplet {

/**
 *  Copyright notice
 *  
 *  This file is part of the Processing sketch `Augmented Reality Pendulum' 
 *  http://www.gwoptics.org/processing/
 *  
 *  Copyright (C) 2011 onwards Ludovico Carbone and Andreas Freise
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 *  MA  02110-1301, USA.
 *
 */

/* Augmented reality pendulum
 * 
 * This sketch uses the web camera video to create an `Augmented Reality Pendulum',
 * i.e. it shows a virtual pendulum and the suspension point of this pendulum
 * follows a special marker whenever this detected in the video stream.
 *
 * This sketch can be used to illustrate how a pendlulum can be utilised as a 
 * seismic attentuation or isolation system (move the suspension point slowly and
 * the pendulum will swing wide, but move the suspension point quickly to the
 * left and right and the lower part of the pendulum will move *less* than the
 * suspension point.
 *
 * The sketch implements the "Pendulum" sketch by E.Schabel based on leapfrog method integration
 * and uses the nyARToolkit package:
 * http://nyatla.jp/nyartoolkit/wiki/index.php?NyARToolkit%20for%20Java.en
 * a good example on how to use it here:
 * http://processing.org/discourse/yabb2/YaBB.pl?num=1271251677
 *
 */




 

public mycapture cap;
FullScreen fs;
PImage logo; // load logo

// MARKER VARIABLES
float mark_x; // marker hor
float mark_y; // marker vert
float mark_z; // marker distance
static float mark_0 = 100; // size of the marker in pixels @ z = 500 (rough), using a 8cm marker
static float mark_r = 0.08f;// size of printed marker in cm

// PENDULUM
// position, velocity, and acceleration variables:
float u, v; // suspension point
float x = 0, y = 0, xx=0, yy=0, xxx=0, yyy=0; // first mass
float a = 0, b = 0, aa=0, bb=0, aaa=0, bbb=0; // secon mass
float d1, d2; // length of wires
static final int start_x=320, start_y=50, start_z=500; // pend suspension coordinates at t = 0

// parameters: accelerations
// note that mass is implicitly ignored, assume m = 1 always
float kx_el1, dkx_el1, ky_el1, dky_el1, b_vx1, b_vy1; // mass 1
float kx_el12, ky_el12; // between mass 1 and 2
float kx_el2, dkx_el2, ky_el2, dky_el2, b_vx2, b_vy2; // mass 2

//pendulum constants
static final float l=mark_0*1.5f; // pendulum length (in units of the marker)
static final float g0 = 9.81f*mark_0/mark_r * 0.25f; // gravitational acceleration (in units of the marker but lowered to 25%)
static final float k=g0 / (0.01f * l); // suspension wire elastic contant
static final float Q_mec = 20; // mechanical quality factor
static final float n=sqrt(g0/l)/Q_mec; // viscous damping
static final float f=sqrt(k)/(Q_mec/5); // wire anelasticity, make the respective Q 5 times lower then for gas damping
static final float maxv=2500; // maximum limit for velocity 

// options for switching to mouse countrol, grab pendulum, change wire elasticity from spring to string-like, display text for options
public boolean bStop;
boolean camON = true, free = true;
boolean[] grab = {
  false,false,false
};
String txt_m, txt_f, txt;

// PLOTTING
// pendulum coordinates
float screen_x; // hor
float screen_y; // vert
float screen_z; // distance from camera

float xp, yp; // first mass
float ap, bp; // second mass
float z_sc, ex_sz, ey_sz; // scaling and dimension of ellipses

// timing
float fps=150; // frame rate, to increase this >100 you need to increase the wait time for the marker detection
float dt;
float timer;

// GRAPHICAL
int w;
int h;

// INITIALIZE MARKER
marker markerthread;

/************************************************************/
public void setup() {

  try { 
    quicktime.QTSession.open();
  } 
  catch (quicktime.QTException qte) { 
    qte.printStackTrace();
  }

  size(640,480);
  w=width;
  h=height;
  frameRate(fps);

  // load gwoptics logo in memory
  logo = loadImage("gwoptics_org_logo_80px.png");

  fs = new FullScreen(this);   
  cap=new mycapture(this,w,h);
  markerthread = new marker(this,cap, 10, w, h, start_x, start_y, start_z);

  //  start face detection thread
  markerthread.start();

  // initialize pend blobs coordinates
  u=start_x;
  v=start_y;
  screen_x = u;
  screen_y = v;
  screen_z = start_z;
  x=u;
  y=v+l;
  a=x;
  b=y+l;

  // initialise timer
  timer=millis(); 
  dt = 1/fps;

  smooth();
}

public void draw() {
  // flip image for mirroring
  scale(-1,1);
  translate(-w,0);

  // display camera image
  markerthread.display(w,h);

  // draw logo and black frame
  bframe(); 
  scale(-1,1);
  int logo_offset;
  if (fs.isFullScreen())
  {
    logo_offset=10;
  }
  else
  {
    logo_offset=0;
  }
  image(logo, -logo.height/20, logo.height/20+logo_offset,-logo.width/3,logo.height/3);  
  scale(-1,1);

  // get hor and vert coordinates for suspension point
  // choose to use marker or mouse to drive pendulum 
  if (camON == true) {
    mark_x=0.5f*markerthread.get_xpos();
    mark_y=0.5f*markerthread.get_ypos();
    mark_z=0.5f*markerthread.get_zpos();
  } 
  else 
  { 
    if (mousePressed == true && mouseButton == LEFT) {
      if (abs(w - mouseX-screen_x) < 3 * ex_sz && abs(mouseY - screen_y) < 3 * ex_sz && grab[1]==false && grab[2]==false )
      {
        mark_x = w - mouseX; 
        mark_y = mouseY;
        grab[0] = true;
      }
      else if (abs(w - mouseX-x) < 1.5f * ex_sz && abs(mouseY - y) < 1.5f * ex_sz  && grab[0]==false && grab[2]==false)
      {
        x = w - mouseX; 
        y = mouseY;
        grab[1] = true;
      }
      else if (abs(w - mouseX-a) < 1.5f * ex_sz && abs(mouseY - b) < 1.5f * ex_sz && grab[0]==false && grab[1]==false)
      {
        a = w - mouseX; 
        b = mouseY;
        grab[2] = true;
      }
    }
    else {
      grab[0]=false;
      grab[1]=false;
      grab[2]=false;
    }
  }

  // add text for options
  pushMatrix();
  textAlign(RIGHT); 
  scale(-1,1); 
  int bright=210;
  int low=120;
  int text_offset;
  if (fs.isFullScreen())
  {
    text_offset=15;
  }
  else
  {
    text_offset=7;
  }    
  if (camON) 
  {
    fill(bright);
    text("marker",-188,h-text_offset);
    fill(low);
    text("mouse",-140,h-text_offset);
  }
  else
  {
    fill(low);
    text("marker",-188,h-text_offset);
    fill(bright);
    text("mouse",-140,h-text_offset);
  }
  if (free == true) 
  {
    fill(low);
    text("spring",-65,h-text_offset);
    fill(bright);
    text("string",-20,h-text_offset);
  }
  else
  {
    fill(bright);
    text("spring",-65,h-text_offset);
    fill(low);
    text("string",-20,h-text_offset);
  }
  popMatrix();

  // smoothing upper bob coordinates
  float easing = 0.2f;
  screen_x = (1-easing) * screen_x + (easing) * mark_x;
  screen_y = (1-easing/2) * screen_y + (easing/2) * mark_y;
  screen_z = (1-easing/4) * screen_z + (easing/4) * mark_z;
  screen_z = max(screen_z,200); // put limit on how close the marker can get to the camera

  // set pend suspension coordinates
  u=screen_x;
  v=screen_y;

  if(millis()-timer>2000) {
    timer=millis();
    fps=frameRate;
    dt = 1/fps;
    println(fps);
  }

  // update change of coordinates according to equation of motion
  x=x+xx*dt;  
  y=y+yy*dt;
  a=a+aa*dt;  
  b=b+bb*dt;

  //calculating the distances between the masses:
  d1=sqrt((x-u)*(x-u)+(y-v)*(y-v));
  d2=sqrt((x-a)*(x-a)+(y-b)*(y-b));

  // calculating accelerations for mass 1:
  // elasticity of wires
  if (free == true) { 
    // string
    kx_el1 = -k*max(d1-l,0)*(x-u)/d1; 
    ky_el1 = -k*max(d1-l,0)*(y-v)/d1; // wire 1
    kx_el12 = k*max(d2-l,0)*(a-x)/d2; 
    ky_el12 = k*max(d2-l,0)*(b-y)/d2; //wire 2
  } 
  else {
    // spring
    kx_el1 = -k*(d1-l)*(x-u)/d1; 
    ky_el1 = -k*(d1-l)*(y-v)/d1; //wire 1
    kx_el12 = k*(d2-l)*(a-x)/d2; 
    ky_el12 = k*(d2-l)*(b-y)/d2; //wire 2
  }

  // anelasticity of wire
  dkx_el1 = -f*(xx*(x-u)+yy*(y-v))*(x-u)/d1/d1;
  dky_el1 = -f*(xx*(x-u)+yy*(y-v))*(y-v)/d1/d1;

  // gas damping
  b_vx1 = -n*xx;
  b_vy1 = -n*yy;

  // summarize acceleration
  xxx = kx_el1 + kx_el12  + b_vx1+ dkx_el1;
  yyy = g0 + ky_el1 + ky_el12  + b_vy1 + dky_el1; // sum of all above accelerations + of course gravity g0

  // calculating accelerations for mass 2:
  // elasticity
  //wire 2
  if (free == true) {
    // string
    kx_el2 = -k*max(d2-l,0)*(a-x)/d2;
    ky_el2 = -k*max(d2-l,0)*(b-y)/d2;
  } 
  else {
    // spring
    kx_el2 = -k*(d2-l)*(a-x)/d2;
    ky_el2 = -k*(d2-l)*(b-y)/d2;
  }

  // anelasticity of wire
  dkx_el2 = -f*(aa*(a-x)+bb*(b-y))*(a-x)/d2/d2;
  dky_el2 = -f*(aa*(a-x)+bb*(b-y))*(b-y)/d2/d2;

  // gas damping
  b_vx2 = -n*aa;
  b_vy2 = -n*bb;

  // summarize acceleration
  aaa = kx_el2 + b_vx2 + dkx_el2; 
  bbb = g0 + ky_el2 + b_vy2 + dky_el2;  // sum of all above accelerations + of course gravity g0

  //calculating velocities (bounded by maxv):
  xx=min(max(xx+xxx*dt,-maxv),maxv);
  yy=min(max(yy+yyy*dt,-maxv),maxv);
  aa=min(max(aa+aaa*dt,-maxv),maxv);
  bb=min(max(bb+bbb*dt,-maxv),maxv);

  // draw pendulum
  z_sc = 500 / screen_z; // scaling with respect to distance z

    // pend drawing coordinates
  xp = screen_x+(x-screen_x) * z_sc;
  yp = screen_y+(y-screen_y) * z_sc;
  ap = screen_x+(a-screen_x) * z_sc;
  bp = screen_y+(b-screen_y) * z_sc;

  // size of blobs  
  ex_sz = 30 * z_sc; // x size of blobs 
  ey_sz = 30 * z_sc; // y size of blobs

  // draw wires
  wire(xp,yp,screen_x,screen_y);
  wire(xp,yp,ap,bp);

  // draw blobs
  bob(screen_x,screen_y,ex_sz);
  bob(xp,yp,ex_sz);
  bob(ap,bp,ex_sz);
}


// pendulum drawings: Wire
public void wire(float x1, float y1, float x2, float y2)
{  
  stroke(105,105,105,255);
  strokeWeight(4);
  line(x1,y1,x2,y2);
  stroke(150);
  strokeWeight(3);
  line(x1,y1,x2,y2);
  stroke(240);
  strokeWeight(1);
  line(x1,y1,x2,y2);
}
// bobs
public void bob(float a, float b, float c) 
{
  stroke(0,0,0,50);
  strokeWeight(c/20);
  fill(255,255,255);
  ellipse(a,b,c,c); 
  noStroke();
}

// black frame
public void bframe() 
{
  noFill();
  stroke(0,0,0,200);
  strokeWeight(width/200);
  rect(0,0,w,h);
  stroke(0,0,0,140);
  strokeWeight(width/100);
  rect(0,0,w,h);
  noStroke();
}

// Keyboard inputs
public void keyPressed()
{    
  // pause marker detection
  if(key == ' ') {
    bStop = !bStop;
    if (bStop) {
      noLoop();
      markerthread.pause();
    } 
    else {
      markerthread.unpause();
      loop();
    }
  }

  // switch between marker and mouse
  if(key == 'm') {
    camON = !camON;
  }
  // switch between spring and string
  if(key == 's') {
    free = !free;
  }
  // stop pendulum and restart
  if(key == 'r') {
    u = screen_x; 
    v = screen_y; // upper bob
    x = u;  
    y = v+l; // intermediate bob
    a = x;  
    b = y+l; // lower bob
    xx = 0; 
    yy = 0; 
    aa = 0; 
    bb = 0; //velocities
  }
}

public void stop() 
{
  markerthread.quit();
  delay(2000);
  cap.stop();
  super.stop();
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#FFFFFF", "AR_pendulum" });
  }
}
