/**
 *  Copyright notice
 *  
 *  This file is part of the Processing sketch `Augmented Reality Pendulum' 
 *  http://www.gwoptics.org/processing/
 *  
 *  Copyright (C) 2011 onwards Ludovico Carbone and Andreas Freise
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 *  MA  02110-1301, USA.
 *
 */

import processing.core.PApplet;
import processing.core.PImage;
import processing.video.*;
import jp.nyatla.nyar4psg.*;

public class marker extends Thread {
  mycapture _camera;
  NyARBoard mark1;
  private boolean running;          
  private int wait;                  
  private int _w;
  private int _h;
  private PApplet parent;
  float mark_x;
  float mark_y;
  float mark_z;
  int detections;
  PImage img;
  boolean paused;
  static float delta_xy=150;
  boolean show;

  public marker (PApplet pa, mycapture cap, int wt, int w, int h, int start_x, int start_y, int start_z) {
    parent=pa; 
    _camera=cap;
    wait = wt;
    running = false;
    _w=w;
    _h=h;
    mark_x=start_x*2;
    mark_y=start_y*2;
    mark_z=start_z*2;
    detections=-10;
    img=_camera.get_image();
    paused=false;
    show=false;
  }

  public void start ()
  {
    mark1=new NyARBoard(parent,_w,_h,"camera_para.dat","gw_marker.patt",80);
    //mark1=new NyARBoard(parent,_w,_h,"camera_para.dat","GW16.pat",80); //new GW marker 
    //mark1=new NyARBoard(parent,_w,_h,"camera_para.dat","gwoptics_marker.pat",80); // gwoptics marker
    mark1.gsThreshold=80;
    mark1.cfThreshold=0.4;
    running = true;
    super.start();
  }

  public float get_xpos() {
    return mark_x;
  }

  public float get_ypos() {
    return mark_y;
  }
  public float get_zpos() {
    return mark_z;
  }

  public void display(int w, int h) {
    parent.image(img,0,0,w,h);
  }

  public void run()
  {
    while (running) {
      if (!paused) {
        img=_camera.get_image();
        if (mark1.detect(img)) {
          float tmp_x = 2 * ((mark1.pos2d[0][0]+mark1.pos2d[1][0]+mark1.pos2d[2][0]+mark1.pos2d[3][0])/4+0);
          float tmp_y = 2 * ((mark1.pos2d[0][1]+mark1.pos2d[1][1]+mark1.pos2d[2][1]+mark1.pos2d[3][1])/4+0);
          float tmp_z = 2 * mark1.trans.z;

          if ((Math.abs(mark_x-tmp_x)<delta_xy && Math.abs(mark_y-tmp_y)<delta_xy) || !show) {
            detections++;
            if (detections>20) {
              mark_x = tmp_x;
              mark_y = tmp_y;
              mark_z = tmp_z;
              show=true;
            }
          }
          else {
            nodetection();
          }
        }
        else {
          nodetection();
        }
      }
      try {
        sleep((long)(wait));
      }
      catch (Exception e) {
      }
    }
  }

  void nodetection() {
    detections--;
    if (detections<-10) {
      detections=-10;
      show=false;
    }
  }

  public int get_detections() {
    return (detections);
  }

  public void pause() {
    paused=true;
  }

  public void unpause() {
    paused=false;
  }


  public void quit()
  {
    running = false;   
    interrupt(); 
    // Now sleep a while to avoid memeory errors 
    try {
      sleep(100l);
    } 
    catch (Exception e) {
    }
  }
}

